@@ -25,18 +25,25 @@
 #define STOPISH_MODE_PAUSE  1
 #define STOPISH_MODE_RESUME 2
 
+#define STOPISH_PORTRAIT  0
+#define STOPISH_LANDSCAPE 1
+
+#define TIMER_PRECISION_HOUR   0
+#define TIMER_PRECISION_MINUTE 1
+
 // stopish.c
 gint dbus_callback( const gchar *interface, const gchar *method,
 	                GArray *arguments, gpointer data, osso_rpc_t *retval );
 int stopish_get_mode( void );
 
 // timer.c
-gint stopish_timeout_cb( gpointer data );
 char *stopish_get_time_string( void );
 long int stopish_get_time_timer( void );
 void stopish_set_time_start( long int time );
 void stopish_timer_resume( void );
 void stopish_timer_save( void );
 long int stopish_current_time( void );
+void stopish_timer_set_precision( int setting );
+int stopish_timer_get_precision( void );
 
 #endif
